@@ -258,11 +258,19 @@ static void device_remove_avdtp_timer(struct audio_device *dev)
 static gboolean headset_connect_timeout(gpointer user_data)
 {
 	struct audio_device *dev = user_data;
+	struct dev_priv *priv = dev->priv;
 
 	dev->priv->headset_timer = 0;
 
-	if (dev->headset)
-		headset_config_stream(dev, FALSE, NULL, NULL);
+	if (dev->headset == NULL)
+		return FALSE;
+
+	if (headset_config_stream(dev, FALSE, NULL, NULL) == 0) {
+		if (priv->state != AUDIO_STATE_CONNECTED &&
+				(priv->sink_state == SINK_STATE_CONNECTED ||
+				priv->sink_state == SINK_STATE_PLAYING))
+			device_set_state(dev, AUDIO_STATE_CONNECTED);
+	}
 
 	return FALSE;
 }
@@ -326,9 +334,16 @@ static void device_sink_cb(struct audio_device *dev,
 		}
 		if (priv->hs_state == HEADSET_STATE_DISCONNECTED)
 			device_set_state(dev, AUDIO_STATE_DISCONNECTED);
-		else if (old_state == SINK_STATE_CONNECTING &&
-				priv->hs_state == HEADSET_STATE_CONNECTED)
-			device_set_state(dev, AUDIO_STATE_CONNECTED);
+		else if (old_state == SINK_STATE_CONNECTING) {
+			switch (priv->hs_state) {
+			case HEADSET_STATE_CONNECTED:
+			case HEADSET_STATE_PLAY_IN_PROGRESS:
+			case HEADSET_STATE_PLAYING:
+				device_set_state(dev, AUDIO_STATE_CONNECTED);
+			default:
+				break;
+			}
+		}
 		break;
 	case SINK_STATE_CONNECTING:
 		device_remove_avdtp_timer(dev);
@@ -397,7 +412,8 @@ static void device_headset_cb(struct audio_device *dev,
 		if (priv->sink_state == AVDTP_SESSION_STATE_DISCONNECTED)
 			device_set_state(dev, AUDIO_STATE_DISCONNECTED);
 		else if (old_state == HEADSET_STATE_CONNECT_IN_PROGRESS &&
-				priv->sink_state == SINK_STATE_CONNECTED)
+				(priv->sink_state == SINK_STATE_CONNECTED ||
+				priv->sink_state == SINK_STATE_PLAYING))
 			device_set_state(dev, AUDIO_STATE_CONNECTED);
 		break;
 	case HEADSET_STATE_CONNECT_IN_PROGRESS:
@@ -406,7 +422,9 @@ static void device_headset_cb(struct audio_device *dev,
 			device_set_state(dev, AUDIO_STATE_CONNECTING);
 		break;
 	case HEADSET_STATE_CONNECTED:
-		if (old_state == HEADSET_STATE_PLAYING)
+		if (old_state == HEADSET_STATE_CONNECTED ||
+				old_state == HEADSET_STATE_PLAY_IN_PROGRESS ||
+				old_state == HEADSET_STATE_PLAYING)
 			break;
 		if (dev->auto_connect) {
 			if (!dev->sink)
@@ -443,7 +461,8 @@ static DBusMessage *dev_connect(DBusConnection *conn, DBusMessage *msg,
 
 	if (dev->headset)
 		headset_config_stream(dev, FALSE, NULL, NULL);
-	else if (dev->sink) {
+
+	if (priv->state != AUDIO_STATE_CONNECTING && dev->sink) {
 		struct avdtp *session = avdtp_get(&dev->src, &dev->dst);
 
 		if (!session)
@@ -455,7 +474,7 @@ static DBusMessage *dev_connect(DBusConnection *conn, DBusMessage *msg,
 		avdtp_unref(session);
 	}
 
-	/* The previous call should cause a call to the state callback to
+	/* The previous calls should cause a call to the state callback to
 	 * indicate AUDIO_STATE_CONNECTING */
 	if (priv->state != AUDIO_STATE_CONNECTING)
 		return g_dbus_create_error(msg, ERROR_INTERFACE
